#include "DesktopName.h"
#include "Config.h"

#include <iostream>

namespace Visibility {

	DesktopName::DesktopName(::Window parent, std::string name) {
		this->name = name;
		is_active = false;
		Font::TextMetrics metrics;
		Config * config = Config::instance();
		config->font->get_metrics(name + ": ", metrics);

		width = metrics.extents.width;
		height = metrics.height;

		window = Window::get_window(0, 0, width, height, parent);
		active.reset(new Pixmap(window->drawable, width, height));
		inactive.reset(new Pixmap(window->drawable, width, height));

		window->select_input(ButtonReleaseMask);

		fill_pixmaps();

		window->set_background_pixmap(inactive);
		window->map();
		window->clear();

		//draw();
	}

	void DesktopName::fill_pixmaps(void) {
		Config * config = Config::instance();
		active->fill_rectangle(config->active_bg_colour, 0, 0, width, height);
		inactive->fill_rectangle(config->inactive_bg_colour, 0, 0, width, height);

		Font::TextMetrics metrics;
		config->font->get_metrics(name + ": ", metrics);

		unsigned baseline = metrics.calculate_baseline(metrics.height);

		active->draw_text((0xff << 24) | config->active_text_colour, config->font, name + ": ", metrics.extents.x, baseline);
		inactive->draw_text((0xff << 24) | config->inactive_text_colour, config->font, name + ": ", metrics.extents.x, baseline);
	}

	void DesktopName::reconfigure(void) {
		Font::TextMetrics metrics;
		Config * config = Config::instance();
		config->font->get_metrics(name + ": ", metrics);

		width = metrics.extents.width;
		height = metrics.height;
		window->resize(width, height);

		active.reset(new Pixmap(window->drawable, width, height));
		inactive.reset(new Pixmap(window->drawable, width, height));

		fill_pixmaps();
		window->set_background_pixmap(is_active ? this->active : this->inactive);

	}

	void DesktopName::get_size(unsigned& width, unsigned& height) {
		width = this->width;
		height = this->height;
	}

	void DesktopName::set_active(bool active) {
		is_active = active;
		window->set_background_pixmap(active ? this->active : this->inactive);
		window->clear();
	}

	DesktopName::~DesktopName() {
		window->unmap();
		window->destroy();
	}

}
