#include "Drawable.h"
#include "Server.h"
#include "Colour.h"

#include <iostream>

namespace Visibility {

bool Drawable::put_image(XImage * image, unsigned src_x, unsigned src_y, unsigned dest_x, unsigned dest_y, unsigned width, unsigned height) {
	Server * server = Server::instance();
	if (XPutImage(server->display, drawable, server->default_gc, image, src_x, src_y, dest_x, dest_y, width, height) == Success) {
		return true;
	}
	return false; // failed
}

bool Drawable::copy_area(Drawable& src, unsigned src_x, unsigned src_y, unsigned dest_x, unsigned dest_y, unsigned width, unsigned height) {
	Server * server = Server::instance();
	if (XCopyArea(server->display, src.drawable, drawable, server->default_gc, src_x, src_y, width, height, dest_x, dest_y) == Success) {
		return true;
	}
	return false; // failed

}

bool Drawable::fill_rectangle(CARD32 colour, unsigned x, unsigned y, unsigned width, unsigned height) {
	Server * server = Server::instance();
	colour = Colour(colour).pixel_value(server->depth);
	XSetForeground(server->display, server->default_gc, colour);
	if(XFillRectangle(server->display, drawable, server->default_gc, x, y, width, height) == Success) {
		return true;
	}
	return false;
}

void Drawable::draw_text(CARD32 colour, Font * font_, std::string text, unsigned x, unsigned y) {
	Server * server = Server::instance();
	XftFont * font = font_->font;

	XftColor xftcolor;
	xftcolor.pixel = colour;
	xftcolor.color.red = (unsigned short)((((colour >> 16) & 0xff) / 255.0) * 65535);
	xftcolor.color.green = (unsigned short)((((colour >> 8) & 0xff) / 255.0) * 65535);
	xftcolor.color.blue = (unsigned short)(((colour & 0xff) / 255.0) * 65535);
	xftcolor.color.alpha = (unsigned short)((((colour >> 24) & 0xff) / 255.0) * 65535);

	if (!draw) {
		draw = XftDrawCreate(server->display, drawable, server->visual, server->colormap);
	}

	XftDrawStringUtf8(draw, &xftcolor, font, x, y, (const FcChar8*)text.c_str(), text.length());

	/*XftFontClose(server->display, font);*/

}

Drawable::~Drawable() {
	if (draw) XftDrawDestroy(draw);
}

void Drawable::get_geometry(int &x, int &y, unsigned &width,
	unsigned &height) {
	Server * server = Server::instance();
	::Window root_return = 0;
	unsigned border_width_return = 0;
	unsigned depth_return = 0;
	XGetGeometry(server->display, drawable, &root_return, &x, &y, &width,
		&height, &border_width_return, &depth_return);
}

}
