// a higher level interface

#ifndef WINDOW_ICON_HEADER_INCLUDED
#define WINDOW_ICON_HEADER_INCLUDED

#include "Window.h"
#include "Pixmap.h"
#include <string>
#include "Tooltip.h"
#include "MainLoop.h"

namespace Visibility {

	class WindowIcon {
	public:
		WindowIcon(::Window parent, ::Window client);
		~WindowIcon();

		inline bool map(void) {return window->map();}

		// the client window
		::Window client;
		// this window (subwindow of main app window)
		std::tr1::shared_ptr<Window> window;

		static Tooltip tooltip;
	protected:
		bool is_active; // if the desktop that this client is on is active
		Window::State _state; // if the client is iconified
		bool window_is_active;

		ARGBData * icon_data;

		void initialize(::Window parent);

		void reconfigure(void); // re-read config (called when config emits signal_changed)
		void draw(void); // sets the window background pixmap depending on values of _state and is_active
		void icon_changed_callback(::Window client); // re-reads the window icon
		void state_changed_callback(::Window client); // when a window becomes iconified or demands attention, this is relevant
		void active_window_changed_callback(void);
		void desktop_changed_callback(void);
		void reconfigure_callback(void);
		void prepare_tooltip(void);
		void pointer_entered_callback(::Window window);
		void clicked_callback(::Window window);

		// used to make it go crazy when the window demands attention
		bool toggle_state(void);

		bool demands_attention_state;

		std::tr1::shared_ptr<MainLoop::FunctorBase> urgent_timeout;

		sigc::connection signal_desktop_changed_connection;
		sigc::connection signal_active_window_changed_connection;
		sigc::connection signal_config_changed_connection;
	};

}

#endif
